//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: CodeWarCommands.h
//
// Description: Common header file with the commands that are sent between
//              the client and the server
//
//*****************************************************************************

#pragma once

#include <utility>
#include "CodeWarTypes.h"

////////////////////////////////////////////////////////////////////
// Info that commands or events can contain

struct StatusInfo
{
    StatusInfo(FoodType f = -1, LifeType l = -1, HeadingType h = North)
    {
        m_food = f;
        m_life = l;
        m_heading = h;
    }

    FoodType m_food;
    LifeType m_life;
    HeadingType m_heading;
};

////////////////////////////////////////////////////////////////////
// The base struct (this will not be sent)

struct BasicCommand
{
    BasicCommand()
    {
        m_version = theVersion;
        m_command = cmdIdle;
    }

    VersionType m_version;
    CommandType m_command;
};

////////////////////////////////////////////////////////////////////
// The commands Server -> Client

struct pollCommand : BasicCommand
{
    pollCommand()
    {
        m_command = cmdPoll;
    }
};

struct infoCommand : BasicCommand
{
    infoCommand(IDType id=-1, StatusInfo s = StatusInfo())
    {
        m_command = cmdInfo;
        m_id = id;
        m_status = s;
    }

    IDType m_id;
    StatusInfo m_status;
};

struct onHitWallCommand : BasicCommand
{
    onHitWallCommand()
    {
        m_command = cmdHitWall;
    }
};

////////////////////////////////////////////////////////////////////
// The commands Client -> Server

struct registerCommand : BasicCommand
{
    registerCommand(AttackType a = -1, DefenseType d = -1, FoodType f = -1)
    {
        m_command = cmdRegister;
        m_attack = a;
        m_defense = d;
        m_food = f;
    }

    AttackType m_attack;
    DefenseType m_defense;
    FoodType m_food;
};

struct idleCommand : BasicCommand
{
    idleCommand()
    {
        m_command = cmdIdle;
    }
};

struct moveForwardCommand : BasicCommand
{
    moveForwardCommand()
    {
        m_command = cmdMoveForward;
    }
};
